//-----------------------------------------------------------------------------
//
// Title       : AESRN
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : AESRN.v
// Generated   : Tue May 12 10:50:34 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This module represents the complete connection between AGERN, ALFERN and
// TCCFBs with inplemented LUTs. See comments for connection guidance.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module AESRN (
	//*** Input wires ***
	
	input wire [339:0] configuration_bits,
	input wire [59:0] ev_c_in,
	input wire [59:0] ev_d_in,
	input wire [63:0] r_e,
	input wire		  rst,
	
	//***Output regs ****
	output reg [63:0] ev_c_out,
	output reg [63:0] ev_d_out,
	output reg [59:0] t_e
	);

	// *** AGERN regs and wires ****
	reg [47:0] AGERN_c_in;
	reg [47:0] AGERN_d_in;
	reg [47:0] AGERN_r_e;
	wire [47:0] AGERN_c_out;
	wire [47:0] AGERN_d_out;
	wire [47:0] AGERN_t_e;
	
	//** ALFERN regs and wires
	reg [27:0] ALF_c_in;
	reg [27:0] ALF_d_in;
	reg [31:0] ALF_r_e;
	wire [31:0] ALF_c_out;
	wire [31:0] ALF_d_out;
	wire [27:0]	ALF_t_e;
	
	
always @ *
	begin: SET_VALUES
		
		//*** AESRN OUtputs ****
		ev_c_out[39:0] = AGERN_c_out[47:8];   //Excluding sw 1 connected to ALFERN
		ev_c_out[47:40] = ALF_c_out[7:0];
		ev_c_out[63:48] = ALF_c_out[31:16];	  //excluding sw 2 connected to AGERN
		ev_d_out[39:0] = AGERN_d_out[47:8];   //Excluding sw 1 connected to ALFERN
		ev_d_out[47:40] = ALF_d_out[7:0];
		ev_d_out[63:48] = ALF_d_out[31:16];	  //excluding sw 2 connected to AGERN
		
		t_e[39:0] = AGERN_t_e[47:8];			//Ecxluding t_e from switch 1
		t_e[47:40] = ALF_t_e[7:0];
		t_e[59:48] = ALF_t_e[27:16];			//Excluding t_e to sw 1
		
		
		// *** AGERN connections ****
		AGERN_c_in[7:0]  = ALF_c_out[15:8]; //From ALFERN global out switch
		AGERN_c_in[47:8] = ev_c_in[39:0];   //From testbench environment
		AGERN_d_in[7:0]  = ALF_d_out[15:8];
		AGERN_d_in[47:8] = ev_d_in[39:0];	  
		
		AGERN_r_e[7:0] = ALF_t_e[15:8];			//From ALFERN global in switch
		AGERN_r_e[47:8] = r_e[39:0]; 			//Testbench signals
		
		// *** ALFERN connections ***
		ALF_c_in[15:8] = AGERN_c_out[7:0];	 //To ALFERN in switch from AGERN switch 2 
		ALF_c_in[7:0]  = ev_c_in[47:40];	 //ALFERN I/O inputs
		ALF_c_in[27:16] = ev_c_in[59:48];
		ALF_d_in[15:8] = AGERN_d_out[7:0];
		ALF_d_in[7:0]  = ev_d_in[47:40];	 //ALFERN I/O inputs
		ALF_d_in[27:16] = ev_d_in[59:48];
		
		ALF_r_e[7:0] = r_e[47:40];       		//To in switch from Port / I/O
		ALF_r_e[15:8] = AGERN_t_e[7:0];			//To out switch from AGERN
		ALF_r_e[31:16] = r_e[63:48];			//From testbench environment
				
	end											  
	
	
	AGERN_port
		AGERN_module (
			.configuration_bits				(configuration_bits[255:0]),
			.ev_c_in						(AGERN_c_in),
			.ev_d_in						(AGERN_d_in),
			.r_e							(AGERN_r_e),
			.rst							(rst),
			.ev_c_out						(AGERN_c_out),
			.ev_d_out						(AGERN_d_out),
			.t_e							(AGERN_t_e)
			);	
			
	 ALFERN_port
	 	ALFERN_module (
		 .configuration_bits				(configuration_bits[339:256]),
		 .ev_c_in_module					(ALF_c_in),
		 .ev_d_in_module					(ALF_d_in),
		 .r_e_module						(ALF_r_e),
		 .rst								(rst),
		 .ev_c_out_module					(ALF_c_out),
		 .ev_d_out_module					(ALF_d_out),
		 .t_e_module						(ALF_t_e)
		 );
		 	
endmodule
