//-----------------------------------------------------------------------------
//
// Title       : AGERN_port
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : AGERN_port.v
// Generated   : Thu Apr 16 10:09:08 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This module connects eight switch blocks to form the Port / T/C side of
// the AGERN. The routing pattern is programmable by a total of 256
// configuration bits. When connecting switches, input channel 0 from North or
//South direction can arbiarily connect to output channel 0 in West or East
//direction. It is not possible to connect input channel 1 
// to output channel 5. See the adress table given in the main report for a
//description of how the input/output and enable signals are adressed 
//to the surrounding environment.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

// **** Local defines ****
`define CONFIG_SIZE			256
`define INPUT_SIZE			48
`define OUTPUT_SIZE			48


module AGERN_port (
// **** INput wires ****
	input wire [`CONFIG_SIZE-1:0] configuration_bits,
	input wire [`INPUT_SIZE-1:0]  ev_c_in,
	input wire [`INPUT_SIZE-1:0]  ev_d_in,
	input wire [`INPUT_SIZE-1:0]  r_e,
	input wire rst,
	
// **** Output wires ****	
	output wire [`OUTPUT_SIZE-1:0] ev_c_out,
	output wire [`OUTPUT_SIZE-1:0] ev_d_out,
	output wire [`INPUT_SIZE-1:0]  t_e
	
	);

// **** Internal wires and registers ****

// - Intenal signals for switch 1 -
wire [7:0] ev_c_out_sw1_E;
wire [7:0] ev_d_out_sw1_E;
wire [7:0] ev_c_out_sw1_W;
wire [7:0] ev_d_out_sw1_W;

// - Internal signals for switch 2 -
wire [7:0] t_e_sw2_S;
wire [7:0] t_e_sw2_N;

// -Internal signals for switch 3 -
wire [7:0] ev_c_out_sw3_E;
wire [7:0] ev_d_out_sw3_E;
wire [7:0] ev_c_out_sw3_W;
wire [7:0] ev_d_out_sw3_W;

// -Internal signals for switch 4 -
wire [7:0] t_e_sw4_S;
wire [7:0] t_e_sw4_N;	

//Internal signals for switch 5 -
wire [7:0] t_e_sw5_S;
wire [7:0] t_e_sw5_N;
wire [7:0] ev_c_out_sw5_E;
wire [7:0] ev_d_out_sw5_E;
wire [7:0] ev_c_out_sw5_W;
wire [7:0] ev_d_out_sw5_W;

// - Internal signals for switch 6 -
wire [7:0] t_e_sw6_S;
wire [7:0] t_e_sw6_N;
wire [7:0] ev_c_out_sw6_E;
wire [7:0] ev_d_out_sw6_E;
wire [7:0] ev_c_out_sw6_W;
wire [7:0] ev_d_out_sw6_W;

//Internal signals for switch 7 -
wire [7:0] t_e_sw7_S;
wire [7:0] t_e_sw7_N;
wire [7:0] ev_c_out_sw7_E;
wire [7:0] ev_d_out_sw7_E;

// - Internal signals for switch 8 -
wire [7:0] t_e_sw8_N;
wire [7:0] ev_c_out_sw8_E;
wire [7:0] ev_d_out_sw8_E;

// **** Connecting the switches to form the AGERN ****

switch_block
	switch_block_1 (
	.configuration_bits		(configuration_bits[31:0]), 
	.ev_c_in_N				(ev_c_in[7:0]),	  	// North side from Port/ I/O 1
	.ev_d_in_N				(ev_d_in[7:0]),
	.ev_c_in_S				(ev_c_in[15:8]),  	//South side from	 Port / I/O 2
	.ev_d_in_S				(ev_d_in[15:8]),
	.r_e_W					(t_e_sw5_N),	  	//West received from switch 5
	.r_e_E					(t_e_sw2_N),		//East received from switch 2
	.rst					(rst),
	.ev_c_out_W				(ev_c_out_sw1_W),  	//West output to switch 5
	.ev_d_out_W				(ev_d_out_sw1_W),
	.ev_c_out_E				(ev_c_out_sw1_E),  	//East output to switch 2
	.ev_d_out_E				(ev_d_out_sw1_E),
	.t_e_N					(t_e[7:0]),		  	// North output to Port / I/O 1
	.t_e_S					(t_e[15:8])		  	// South output to Port / I/O 2
	);
	
switch_block
	switch_block_2 (
	.configuration_bits		(configuration_bits[63:32]), 
	.ev_c_in_N				(ev_c_out_sw1_E),	// North side from Switch 1
	.ev_d_in_N				(ev_d_out_sw1_E),
	.ev_c_in_S				(ev_c_out_sw6_E),  	//South side from switch 6
	.ev_d_in_S				(ev_d_out_sw6_E),
	.r_e_W					(r_e[7:0]),	  		//West received from Port / I/O 1
	.r_e_E					(r_e[15:8]),		//East received from Port / I/O 2
	.rst					(rst),
	.ev_c_out_W				(ev_c_out[7:0]),  	//West output to Port / I/O 1
	.ev_d_out_W				(ev_d_out[7:0]),
	.ev_c_out_E				(ev_c_out[15:8]),  	//East output to Port / I/O 2
	.ev_d_out_E				(ev_d_out[15:8]),
	.t_e_N					(t_e_sw2_N),		// North output to Switch 1
	.t_e_S					(t_e_sw2_S)		  	// South output to Switch 6
	);
	
switch_block
	switch_block_3 (
	.configuration_bits		(configuration_bits[95:64]), 
	.ev_c_in_N				(ev_c_in[23:16]),	// North side from Port/ I/O 3
	.ev_d_in_N				(ev_d_in[23:16]),
	.ev_c_in_S				(ev_c_in[31:24]),  	//South side from Port / I/O 4
	.ev_d_in_S				(ev_d_in[31:24]),
	.r_e_W					(t_e_sw5_S),	  	//West received from switch 5
	.r_e_E					(t_e_sw4_N),		//East received from switch 4
	.rst					(rst),
	.ev_c_out_W				(ev_c_out_sw3_W),  	//West output to switch 5
	.ev_d_out_W				(ev_d_out_sw3_W),
	.ev_c_out_E				(ev_c_out_sw3_E),  	//East output to switch 4
	.ev_d_out_E				(ev_d_out_sw3_E),
	.t_e_N					(t_e[23:16]),		// North output to Port / I/O 3
	.t_e_S					(t_e[31:24])		// South output to Port / I/O 4
	);
	
 switch_block
	switch_block_4 (
	.configuration_bits		(configuration_bits[127:96]), 
	.ev_c_in_N				(ev_c_out_sw3_E),	// North side from Switch 3
	.ev_d_in_N				(ev_d_out_sw3_E),
	.ev_c_in_S				(ev_c_out_sw6_W),  	//South side from switch 6
	.ev_d_in_S				(ev_d_out_sw6_W),
	.r_e_W					(r_e[23:16]),	  	//West received from Port / I/O 3
	.r_e_E					(r_e[31:24]),		//East received from Port / I/O 4
	.rst					(rst),
	.ev_c_out_W				(ev_c_out[23:16]),  //West output to Port / I/O 3
	.ev_d_out_W				(ev_d_out[23:16]),
	.ev_c_out_E				(ev_c_out[31:24]),  //East output to Port / I/O 4
	.ev_d_out_E				(ev_d_out[31:24]),
	.t_e_N					(t_e_sw4_N),		// North output to Switch 3
	.t_e_S					(t_e_sw4_S)		  	// South output to Switch 6
	);

switch_block
	switch_block_5 (
	.configuration_bits		(configuration_bits[159:128]), 
	.ev_c_in_N				(ev_c_out_sw1_W),	// North side from Switch 1
	.ev_d_in_N				(ev_d_out_sw1_W),
	.ev_c_in_S				(ev_c_out_sw3_W),  	//South side from switch 3
	.ev_d_in_S				(ev_d_out_sw3_W),
	.r_e_W					(t_e_sw7_N),	  	//West received from Switch 7
	.r_e_E					(t_e_sw6_N),		//East received from Switch 6
	.rst					(rst),
	.ev_c_out_W				(ev_c_out_sw5_W),  //West output to Switch 7
	.ev_d_out_W				(ev_d_out_sw5_W),
	.ev_c_out_E				(ev_c_out_sw5_E),  //East output to Switch 6
	.ev_d_out_E				(ev_d_out_sw5_E),
	.t_e_N					(t_e_sw5_N),		// North output to Switch 1
	.t_e_S					(t_e_sw5_S)		  	// South output to Switch 3
	);	

	switch_block
	switch_block_6 (
	.configuration_bits		(configuration_bits[191:160]), 
	.ev_c_in_N				(ev_c_out_sw5_E),	// North side from Switch 5
	.ev_d_in_N				(ev_d_out_sw5_E),
	.ev_c_in_S				(ev_c_out_sw8_E),  	//South side from switch 8
	.ev_d_in_S				(ev_d_out_sw8_E),
	.r_e_W					(t_e_sw4_S),	  	//West received from Switch 4
	.r_e_E					(t_e_sw2_S),		//East received from Switch 2
	.rst					(rst),
	.ev_c_out_W				(ev_c_out_sw6_W),  //West output to Switch 4
	.ev_d_out_W				(ev_d_out_sw6_W),
	.ev_c_out_E				(ev_c_out_sw6_E),  //East output to Switch 2
	.ev_d_out_E				(ev_d_out_sw6_E),
	.t_e_N					(t_e_sw6_N),		// North output to Switch 5
	.t_e_S					(t_e_sw6_S)		  	// South output to Switch 8
	);
	
switch_block
	switch_block_7 (
	.configuration_bits		(configuration_bits[223:192]), 
	.ev_c_in_N				(ev_c_out_sw5_W),	// North side from Switch 5
	.ev_d_in_N				(ev_d_out_sw5_W),
	.ev_c_in_S				(ev_c_in[47:40]),  	//South side from Port A/B
	.ev_d_in_S				(ev_d_in[47:40]),
	.r_e_W					(r_e[39:32]),	  	//West received from GLOBAL
	.r_e_E					(t_e_sw8_N),		//East received from switch 8
	.rst					(rst),
	.ev_c_out_W				(ev_c_out[39:32]),  //West output to GLOBAL
	.ev_d_out_W				(ev_d_out[39:32]),
	.ev_c_out_E				(ev_c_out_sw7_E),  	//East output to switch 8
	.ev_d_out_E				(ev_d_out_sw7_E),
	.t_e_N					(t_e_sw7_N),		// North output to Switch 5
	.t_e_S					(t_e[47:40])		// South output to Port A/B
	);
	
switch_block
	switch_block_8 (
	.configuration_bits		(configuration_bits[255:224]), 
	.ev_c_in_N				(ev_c_out_sw7_E),	// North side from Switch 7
	.ev_d_in_N				(ev_d_out_sw7_E),
	.ev_c_in_S				(ev_c_in[39:32]),  	//South side from GLOBAL
	.ev_d_in_S				(ev_d_in[39:32]),
	.r_e_W					(r_e[47:40]),	  	//West received from PORT A/B
	.r_e_E					(t_e_sw6_S),		//East received from Switch 6
	.rst					(rst),
	.ev_c_out_W				(ev_c_out[47:40]),  //West output to PORT A/B
	.ev_d_out_W				(ev_d_out[47:40]),
	.ev_c_out_E				(ev_c_out_sw8_E),  	//East output to switch 6
	.ev_d_out_E				(ev_d_out_sw8_E),
	.t_e_N					(t_e_sw8_N),		// North output to Switch 7
	.t_e_S					(t_e[39:32])		// South output to GLOBAL
	);
	
`undef CONFIG_SIZE			
`undef INPUT_SIZE			
`undef OUTPUT_SIZE			
	
endmodule
