//-----------------------------------------------------------------------------
//
// Title       : copy_element
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : vopy_element.v
// Generated   : Fri Apr 17 15:08:18 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This module descirbes a copy element used to fork a signal to different
//destinations, and make sure that the correct handshake is sent to and
//received from all sources. It is constructed of C-elments providing
//the stateholding functionalities needed to wait for the response of all
//sources. When the correct handshake response has been received, one 
//signal is sent to the original source of the forked signal ending the 
//transmission.	The copy element is constructed for use in the ALFERN, and
//is therefor constructed to copy four inputs to eight destinations.  
//The one input - two output copy element described in the main report
//is the basic element, and the ALFERN copy element is constructed of four
//basic copy elements.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module copy_element (
	
// **** Input wires ****	
	input wire [3:0] configuration_bits,
	input wire [3:0] ev_c_in_CE,
	input wire [3:0] ev_d_in_CE,
	input wire [7:0] r_e_CE,	   //Received from forked destinations
	
// **** Output wires and registers ****
	output reg [7:0] ev_c_out_CE,
	output reg [7:0] ev_d_out_CE,
	output reg [3:0] t_e_CE		//Enable signal to peripherals	
	);
 
// **** Local wires and registers ****

wire [3:0] c_element_out;   		//From receiving C-elements

// - Generate and connect the C-elements

generate
	genvar g_i;
	for (g_i = 0; g_i < 4; g_i = g_i +1 )
		begin: GEN_C
			if ( g_i < 4 )
				begin
					c_element
						c_element_i
						(
							.a_in	(r_e_CE[g_i]),
							.b_in	(r_e_CE[g_i+4]),
							.s_out	(c_element_out[g_i])
						);
				end		
  		end
endgenerate

// This is the MUX control part, where the configuration bit determines
//if one or two receivers communicate with the each two-input Copy Element.
// If a configuration bit is "1" the corresponding 2-input Copy element
// communicates with two receivers. If the conf. bit is "0" only one
// source is connected.
always @ (configuration_bits, c_element_out, ev_c_in_CE, ev_d_in_CE )
	begin: SELECT_SOURCE
		if ( configuration_bits[0] == 1 )
			begin
				t_e_CE[0] = c_element_out[0];
			end
		else
			begin
				t_e_CE[0] = r_e_CE[0];
			end
		
		if ( configuration_bits[1] == 1 )
			begin
				t_e_CE[1] = c_element_out[1];
			end
		else
			begin
				t_e_CE[1] = r_e_CE[1];
			end
		
		if ( configuration_bits[2] == 1 )
			begin
				t_e_CE[2] = c_element_out[2];
			end
		else
			begin
				t_e_CE[2] = r_e_CE[2];
			end 
			
		if ( configuration_bits[3] == 1 )
			begin
				t_e_CE[3] = c_element_out[3];
			end
		else
			begin
				t_e_CE[3] = r_e_CE[3];
			end 	
			
// Assigning the desired input to the forked outputs.
// The forking is in accord with the architectural drawing featured
//in Appendix A of the main report.
		ev_c_out_CE[0] = ev_c_in_CE[0];	
		ev_d_out_CE[0] = ev_d_in_CE[0];	
		ev_c_out_CE[1] = ev_c_in_CE[1];	
		ev_d_out_CE[1] = ev_d_in_CE[1];	
		ev_c_out_CE[2] = ev_c_in_CE[2];	
		ev_d_out_CE[2] = ev_d_in_CE[2];	
		ev_c_out_CE[3] = ev_c_in_CE[3];	
		ev_d_out_CE[3] = ev_d_in_CE[3];
		ev_c_out_CE[4] = ev_c_in_CE[0];	
		ev_d_out_CE[4] = ev_d_in_CE[0];	
		ev_c_out_CE[5] = ev_c_in_CE[1];	
		ev_d_out_CE[5] = ev_d_in_CE[1];	
		ev_c_out_CE[6] = ev_c_in_CE[2];	
		ev_d_out_CE[6] = ev_d_in_CE[2];	
		ev_c_out_CE[7] = ev_c_in_CE[3];	
		ev_d_out_CE[7] = ev_d_in_CE[3];

	end


endmodule
