//-----------------------------------------------------------------------------
//
// Title       : switch_block_test
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : switch_block_test.v
// Generated   : Wed Apr 15 13:55:25 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This testbench reads a switch block configuration from file, and programmes
//the switch block accordingly. The testbench interacts with the switch block
//according to the 4-PDR handshake protocol, and uses a clock signal
//to synchronize the data sent into the switch. The clock is for input
//synchronization only, and will not interfere with the asynchronous transfer
//scheme in any way. 
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module switch_block_test ();

	parameter ClockPeriod = 20;

// **** Input registers to the test module ****	
	reg [31:0] configuration_bits;
	reg [7:0]  ev_c_in_N;
	reg [7:0]  ev_d_in_N; 
	reg [7:0]  ev_c_in_S;
	reg [7:0]  ev_d_in_S;
	reg [7:0]  t_e_test_W;
	reg [7:0]  t_e_test_E;
	reg rst;
	reg clk;

// **** Output wire from the test module ****	
	wire [7:0]  ev_c_out_W;
	wire [7:0]  ev_d_out_W; 
	wire [7:0]  ev_c_out_E;
	wire [7:0]  ev_d_out_E;
	wire [7:0]  r_e_test_N;
	wire [7:0]  r_e_test_S;

// - Register memory to hold input data from file -

reg [32:1] Test_vectors[8:1];	//One testvector for Each WCHB (configuration)
integer counter;	//To read different inputs from file
reg update_stimulus;
reg shift_conf;
reg	 [7:0] wchb_free_N;		//r_e_test
reg	 [7:0] wchb_free_S;
reg  [7:0] wchb_notify_S;	
reg  [7:0] wchb_notify_N;
reg	 [7:0] wchb_free_W;
reg	 [7:0] wchb_free_E;
reg first_pass;
integer i,n,m,k,l,o;
//- Instanciation of switch block model -

switch_block 
	switch_block_tested ( 
     	.configuration_bits	(configuration_bits),
		.ev_c_in_N			(ev_c_in_N),
		.ev_d_in_N			(ev_d_in_N),
		.ev_c_in_S			(ev_c_in_S),
		.ev_d_in_S			(ev_d_in_S),
		.r_e_W				(t_e_test_W),
		.r_e_E				(t_e_test_E),
		.rst				(rst),		 
		.ev_c_out_W			(ev_c_out_W),
		.ev_d_out_W			(ev_d_out_W),
		.ev_c_out_E			(ev_c_out_E),
		.ev_d_out_E			(ev_d_out_E),
		.t_e_N				(r_e_test_N),
		.t_e_S				(r_e_test_S)
	); 
	
initial clk = 1;
initial rst = 1;
 always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			 
			 $readmemb("SwitchBlockConf.vec",Test_vectors); //read input vectors
// - Initial values of input signals to switch block module -
			 clk = 0;
			 rst = 0;
			 t_e_test_W = 8'b 00000000;
			 t_e_test_E = 8'b 00000000;
			 ev_c_in_N = 8'b 00000000;
			 ev_d_in_N = 8'b 00000000;
			 ev_c_in_S = 8'b 00000000;
			 ev_d_in_S = 8'b 00000000;
			 counter = 0;
			 shift_conf = 1'b 0;
			 wchb_free_W = 8'b 00000000;
			 wchb_free_E = 8'b 00000000;
			 wchb_free_N = 8'b 00000000;
			 wchb_free_S = 8'b 00000000;
			 wchb_notify_N = 8'b 00000000;
			 wchb_notify_S = 8'b 00000000; 
			 first_pass = 1'b 0;
			 
// - Reads the first testvector set from the SwitchBlockConf.vec file. -
//- The testvectors are used to program the switch block- -
			 configuration_bits = Test_vectors[1];
			 
		 	# ClockPeriod rst = 1;

// - Using the testvectors to construct the configuration sequence -		 
			 			 
		end	
 
					
always @ (shift_conf, first_pass,r_e_test_N,r_e_test_S)
	begin: CHECK_AVAILABILITY
	// - The first loop iterates through all t_e signals from all WCHBs (r_e to
	// the testbench),, and marks the ones that are free for data transfer in
	// each of the switch block input directions -
	if( shift_conf == 1 | first_pass == 0 )
		begin
					for( i = 0; i < 8; i = i+1)
						begin: CHECK_WCHBs
							if( r_e_test_N[i] == 1 )
								begin
									wchb_free_N[i] <= 1'b 1;			
								end
							else
								begin
									wchb_free_N[i] <= 1'b 0;
								end
							if( r_e_test_S[i] == 1 )
								begin
									wchb_free_S[i] <= 1'b 1;
								end
							else
								begin
								wchb_free_S[i] <= 1'b 0;
								end
								
							end
						shift_conf = 1'b 0;
		end	
		end		
	// - This loop iterates through all of the wchb_free variables in each of
	//the switchs input directions, and sends a positive event bit on all
	// free transmission paths - 
	always @ (posedge clk)
		begin	
			
			if (rst)
				begin 
		
					for ( o = 0; o < 8; o = o+1 )
						begin: SEND_DATA
							if ( wchb_free_S[o] == 1 )
								begin
									ev_c_in_S[o] <= 1'b 1;
									ev_d_in_S[o] <= 1'b 0;
									wchb_notify_S[o] <= 1'b 1;
								end
							 if ( wchb_free_N[o] == 1 )
								begin
									ev_c_in_N[o] <= 1'b 1;
									ev_d_in_N[o] <= 1'b 0;
									wchb_notify_N[o] <= 1'b 1;
								end
						 end
	
	// - An event bit is send through the switch, and received by the testbench.
	//To make a correct handshake, the TB sets its transmitted enable signal
	//to "0", letting the sending WCHB know it is busy. -
					for( m = 0; m < 8; m = m+1 )
						begin: START_HS
							if  (ev_c_out_W[m] == 1 | ev_d_out_W[m] == 1 )
								begin
									t_e_test_W[m] <= 1'b 0;
									wchb_free_W[m] <= 1'b 0;
								end
							if	(ev_c_out_E[m] == 1 | ev_d_out_E[m] == 1 )
								begin
									t_e_test_E[m] <= 1'b 0;
									wchb_free_E[m] <= 1'b 0;
								end
						end	
	// -The WCHB which has issued an event bit to the TB is sent the appropriate
	// spacer in order to complete the handshake. This spacer means resetting
	// for another reception.
					for (k = 0; k < 8; k = k+1 )
						begin: SET_SPACER
							if ( r_e_test_N[k] == 0 & wchb_notify_N[k] == 1)
								begin
									ev_c_in_N[k] <= 1'b 0;
									ev_d_in_N[k] <= 1'b 0;
									wchb_notify_N[k] <= 0;
								end
							if ( r_e_test_S[k] == 0 & wchb_notify_S[k] == 1)
								begin
									ev_c_in_S[k] <= 1'b 0;
									ev_d_in_S[k] <= 1'b 0;
									wchb_notify_S[k] <= 0;
								end
						end
	// - The final handshake procedure is issued in this loop, where the testbench
	//sets its transmitted enable signal to "1" if a spacer is received from
	//the correct WCHB element. The handshake is complete. This is done for both
	//eastern and western outputs.
					for (l = 0; l < 8; l = l+1 ) 
						begin: RESET_HS
							if ( ev_c_out_W[l] == 0 & ev_d_out_W[l] == 0 )
								begin
									if( wchb_free_W[l] == 0)
										begin
											t_e_test_W[l] <= 1'b 1;
											wchb_free_W[l] <= 1'b 1;
											
										end
								end
							if ( ev_c_out_E[l] == 0 & ev_d_out_E[l] == 0 )
								begin											   
									if( wchb_free_E[l] == 0)
										begin
											t_e_test_E[l] <= 1'b 1;
											wchb_free_E[l] <= 1'b 1;
											
										end
								end
							
									
						end
	
	// - For each clock cycle the transmitted enable signal from the testbench
	//is updated. -
					for( n = 0; n < 8; n = n+1)
						begin: SET_AVAILABILITY
							if ( wchb_free_E[n] == 1 )
								begin
									t_e_test_E[n] <= 1'b 1;
								end
							 if (wchb_free_W[n] == 1)
								 begin
									t_e_test_W[n] <= 1'b 1;
								end
							first_pass = 1;
						end		
				
			end		
	end
	
		
endmodule
