//-----------------------------------------------------------------------------
//
// Title       : AESRN_test
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : AESRN_test.v
// Generated   : Tue May 12 11:51:55 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// Testbench for the AESRN . COnfigurations for two different test scenarios,
// the ones described in the main report, is included.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module AESRN_test ();
	
	parameter ClockPeriod = 20;
	
	reg [339:0] configuration_bits;
	reg [59:0] ev_c_in;
	reg [59:0] ev_d_in;
	reg [63:0] t_e_test;
 	reg		   rst;
	reg 	   clk;
	 
	wire [63:0] ev_c_out;
	wire [63:0] ev_d_out;
	wire [59:0] r_e_test;
	
	
	reg [32:1] Conf_vectors[10:1];	//For switch configurations
	reg [4:1] CE_vectors[3:1];		//Copy element configurations
	reg [8:1] TCCFB_vector [1:1];	// TCCFB configuration
	
	initial clk = 1;
	initial rst = 1;
	
	
	AESRN
		AESRN_module_inst (
			.configuration_bits		(configuration_bits),
			.ev_c_in				(ev_c_in),
			.ev_d_in				(ev_d_in),
			.r_e					(t_e_test),
			.rst					(rst),
			.ev_c_out				(ev_c_out),
			.ev_d_out				(ev_d_out),
			.t_e					(r_e_test)
		);
	
	always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			 clk = 0;
			 rst = 0;
			 t_e_test = 64'b 0000000000000000000000000000000000000000000000000000000000000000;
			 ev_c_in = 60'b 000000000000000000000000000000000000000000000000000000000000;
			 ev_d_in = 60'b 0000000000000000000000000000000000000000000000000000000000;
			 
			 
			 $readmemb("AESRNConf.vec",Conf_vectors); 	//read input vectors to switches
			 $readmemb("CEConf.vec",CE_vectors);   		//read input vectors to CEs
			 $readmemb("TCCFBConf.vec",TCCFB_vector); 	//read input vectors to TCCFB
			 
			 
			//AGERN configuration	 
			configuration_bits[31:0] = Conf_vectors[1];   		//AGERN switch 1
			configuration_bits[63:32] = Conf_vectors[2];		//AGERN switch 2
			configuration_bits[95:64] = Conf_vectors[3]; 		//AGERN switch 3
			configuration_bits[127:96] = Conf_vectors[4]; 		//AGERN switch 4
			configuration_bits[159:128] = Conf_vectors[5]; 		//AGERN switch 5
			configuration_bits[191:160] = Conf_vectors[6];  	//AGERN switch 6
			configuration_bits[223:192] = Conf_vectors[7];		//AGERN switch 7
			configuration_bits[255:224] = Conf_vectors[8];	    //AGERN switch 8
			 
			 configuration_bits[287:256] = Conf_vectors[9];		//ALFERN in switch
			 configuration_bits[319:288] = Conf_vectors[10];  	//ALFERN out switch
			 configuration_bits[323:320] = CE_vectors[1];		//Copy element
			 configuration_bits[327:324] = CE_vectors[2];	 	//Copy element
			 configuration_bits[331:328] = CE_vectors[3];	 	//Copy element
			 configuration_bits[339:332] = TCCFB_vector[1];	 	// TCCFB with LUT
			 
			 # ClockPeriod rst = 1;
			 
			end 
	   
always @ (posedge clk)
	begin: TEST_STIMULUS
	
	// - Local registers -
	reg	 [59:0] module_free = 59'b 00000000000000000000000000000000000000000000000000000000000;			//r_e_test
	reg  [59:0] notify_test = 59'b 00000000000000000000000000000000000000000000000000000000000;			//t_e_test;		//t_e_test
	reg	 [59:0] test_free =   59'b 11111111111111111111111111111111111111111111111111111111111;
	integer i,n,m,k,l,o;    //For couting purposes
	
		if (rst)
			begin						
//Marking all event channels ready to receive inputs 				
				 for (o = 0; o <  64; o = o+1 )   
				   begin: CHECK_STATUS
					   if( test_free[o] == 1)
						   begin
							   if (o > 48 & o <= 55 )	//For duplicate copy oCE outputs
								   begin
									   t_e_test[o+8] = 1'b 1;
									   t_e_test[o] = 1'b 1;
								   end
							else
								t_e_test[o] = 1'b 1;
								
							end
							   
					end	
				
				
// - For each iteration all modules that are transmitting the enable
//signals making them available for data reception will be marked as free.
				for( i = 0; i < 60; i = i+1)
					begin: CHECK_INST
						if( r_e_test[i] == 1 )
							begin
							module_free[i] = 1'b 1;			
							end
						else  
							begin
							module_free[i] = 1'b 0;
							end
					end				
//This if sets up a specific distributions scenario. 40 Issues a signal
//on channel 0 from PORT AB. 17 issues a signal on ch 1 from PORT E
//and 

//*********Test scenario 1 ***** Signals from Port E and 
//Port A/B and ALFERN low CE
		//if (module_free[32] == 1 & module_free[9] == 1 & module_free[48] == 1 ) 
//					begin
//					ev_c_in[32] = 1'b 1;	   //Selected AGERN input
//					ev_d_in[32] = 1'b 0;
//					notify_test[32] = 1'b 1;				  
//					ev_c_in[9] = 1'b 1; 	
//					ev_d_in[9] = 1'b 0;
//					ev_c_in[48] = 1'b 1;		//ALFERN Input
//					ev_d_in[48] = 1'b 0;
//					notify_test[9] = 1'b 1;
//					notify_test[48] = 1'b 1;
//					end	

//*********Test scenario 2 *****
//This is test scenario 2, only using ALFERN inputs. Two from CElow and 1 from CE high.

				if (module_free[54] == 1 & module_free[50] == 1 & module_free[48] == 1 ) 
					begin
					ev_c_in[54] = 1'b 1;	   
					ev_d_in[54] = 1'b 0;
					notify_test[54] = 1'b 1;				  
					ev_c_in[50] = 1'b 1; 	
					ev_d_in[50] = 1'b 0;
					ev_c_in[48] = 1'b 1;		//ALFERN Input
					ev_d_in[48] = 1'b 0;
					notify_test[50] = 1'b 1;
					notify_test[48] = 1'b 1;
					end	
									
// -  Setting the correct handshake while providing a compensation for the fact
//that there are more inputs than outputs to the module. This is due to the
//copy elements.
				for (n = 0; n < 60; n = n+1 ) 
	            	begin: HANDSHAKE
						if( r_e_test[n] == 0 & notify_test[n] == 1)
							begin							
										ev_c_in[n] = 1'b 0;
										ev_d_in[n] = 1'b 0;
										notify_test[n] = 1'b 0;
							end
					end
//Correct handshake taking forking of signals from copy elements under
//consideration.
				for (m = 0; m <  64; m = m+1 ) 
	            	begin: END_HS
						if ( ev_c_out[m] == 0 & ev_d_out[m] == 0)
							begin
								if (m > 55)
									begin
										if (test_free[m-8] == 0 )
											begin
												t_e_test[m] = 1'b 1;
												test_free[m-8] = 1'b 1;
											end
									end
								else if (m < 48)
									begin
										if(test_free[m] == 0)
											begin
												t_e_test[m] = 1'b 1;
												test_free[m] = 1'b 1;
											end
									end
							end
					end
	//Initiating the handshake by acknowleding data reception				
			   for (k = 0; k <  64; k = k+1 ) 
				 	begin: INITIATE_HS
						if (ev_c_out[k] == 1 | ev_d_out[k] == 1 )
							begin
								if ( k > 55 )
									begin
										t_e_test[k] = 1'b 0;
										t_e_test[k-8] = 1'b 0;
										test_free[k-8] = 1'b 0;			
									end
									
							     if (k < 48)
									begin
										t_e_test[k] = 1'b 0;
										test_free[k] = 1'b 0;		
									end		
							end   
					end
					
			  			
			end		
		end				

	

endmodule
