//-----------------------------------------------------------------------------
//
// Title       : TCCFB_LUT_test
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : TCCFB_LUT_test.v
// Generated   : Tue Apr 21 11:16:02 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// SImple testbench for LUT functionality
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module TCCFB_LUT_test ();
	
	parameter ClockPeriod = 20;
	
	reg [7:0] configuration_bits;
	reg [2:0] ev_c_in_LUT;
	reg [2:0] ev_d_in_LUT;
	reg 	  t_e_test;
	reg       rst;
	reg 	  clk;

	wire       ev_c_out_LUT;
	wire       ev_d_out_LUT;
	wire [2:0] r_e_test;

	
	reg [8:1] Configuration_vector[1:1];	//Configuration vector for the Copy Element
	reg asynch_test;
	// - Local registers for test purposes -
	reg	  LUT_free;		
	reg   test_free;	
	integer	 counter;
	reg not_zero;
	reg second_hs;

	
// - Instanciating test module - 	
	TCCFB_LUT
		LUT_test (
			.configuration_bits			(configuration_bits),
			.ev_c_in_LUT				(ev_c_in_LUT),
			.ev_d_in_LUT				(ev_d_in_LUT),
			.r_e_LUT					(t_e_test),	  
			.rst						(rst),
			.ev_c_out_LUT				(ev_c_out_LUT),
			.ev_d_out_LUT				(ev_d_out_LUT),
			.t_e_LUT					(r_e_test)
		);
	
initial clk = 1;
initial rst = 1;
 always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			 
			 $readmemb("LUTConf.vec",Configuration_vector); //read input vectors
// - Initial values of input signals to copy element under test -
			 clk = 0;
			 rst = 0;
			 asynch_test = 1'b 1;
			 t_e_test = 1'b 0;
			 ev_c_in_LUT = 3'b 000;
			 ev_d_in_LUT = 3'b 000;
			 LUT_free = 1'b 0;		
				test_free = 1'b 1;	
				counter = 0;
				not_zero = 1'b 0;
				asynch_test = 1'b 1;
				second_hs = 1'b 0;
// - Reads LUT configuration from the LUTConf.vec file. -

			 configuration_bits = Configuration_vector[1];
			 
		 	# ClockPeriod rst = 1;	
end	

always @ (asynch_test, r_e_test,ev_d_out_LUT,ev_c_out_LUT)
	begin: ASYNCH_TEST 	
		
		
		if ( asynch_test == 1'b 0)
			begin 
				
				if (ev_c_out_LUT == 0 & ev_d_out_LUT == 0 & test_free == 0) 
					begin
					if (test_free == 0)
						begin
							t_e_test = 1'b 1;
							test_free = 1'b 1;
							asynch_test = 1'b 1;  //Handshake complete
						end 
					end 
		
				if ((ev_c_out_LUT == 1'b 1 | ev_d_out_LUT == 1'b 1) & r_e_test == 3'b 000 ) 
					begin
						t_e_test =  1'b 0;
						test_free = 1'b 0;
						second_hs = 1'b 1;
					end 	
				
				if (r_e_test == 3'b 000 & LUT_free == 0 & second_hs) 
					begin
						LUT_free = 1;
						ev_d_in_LUT = 3'b 000;
						ev_c_in_LUT = 3'b 000;
						second_hs = 0;
					end
					
			end
	
	end

always @ (posedge clk)
	begin: TEST_STIMULUS
	
	if (rst & asynch_test)
		begin
//If all LUT inputs are free, a "1" is issued to the testbench.			
			if ( r_e_test == 3'b 111 )
				begin
					LUT_free = 1'b 1;
				end
				
			if ( test_free == 1'b 1 )
				begin
					t_e_test = 1'b 1;
					not_zero = 1'b 1;
				end
			
//Different test scenarios for the LUT. According to the standard configuration used in
//the presented example the LUT shall only issue a "1" when all clock inputs are "1".
			if ( LUT_free == 1'b 1 & counter < 8 & test_free == 1'b 1 & asynch_test == 1 )
				begin
			
				casez (counter) 
					0:	
					 	begin
							ev_c_in_LUT[0] <= 0;
							ev_d_in_LUT[0] <= 1;
							ev_c_in_LUT[1] <= 1;
							ev_d_in_LUT[1] <= 0; 
							ev_c_in_LUT[2] <= 0;
							ev_d_in_LUT[2] <= 1;
							asynch_test = 0;
						end				
					 1:
					 	begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 0;
							ev_d_in_LUT[1] <= 1; 
							ev_c_in_LUT[2] <= 0;
							ev_d_in_LUT[2] <= 1;
							asynch_test <= 0;
						end
					2:	 
						begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 1;
							ev_d_in_LUT[1] <= 0; 
							ev_c_in_LUT[2] <= 0;
							ev_d_in_LUT[2] <= 1;
							asynch_test <= 0;
						end
					3:	 
						begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 1;
							ev_d_in_LUT[1] <= 0; 
							ev_c_in_LUT[2] <= 0;
							ev_d_in_LUT[2] <= 1;
							asynch_test = 0;
						end
					4:
						begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 1;
							ev_d_in_LUT[1] <= 0; 
							ev_c_in_LUT[2] <= 1;
							ev_d_in_LUT[2] <= 0;
							asynch_test = 0;
						end
					5:	
						begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 0;
							ev_d_in_LUT[1] <= 1; 
							ev_c_in_LUT[2] <= 1;
							ev_d_in_LUT[2] <= 0;
							asynch_test = 0;
						end
					6:	 
						begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 1;
							ev_d_in_LUT[1] <= 0; 
							ev_c_in_LUT[2] <= 1;
							ev_d_in_LUT[2] <= 0;
							asynch_test = 0;
						end
					7:	  
						begin
							ev_c_in_LUT[0] <= 1;
							ev_d_in_LUT[0] <= 0;
							ev_c_in_LUT[1] <= 1;
							ev_d_in_LUT[1] <= 0; 
							ev_c_in_LUT[2] <= 1;
							ev_d_in_LUT[2] <= 0;
							asynch_test = 0;
						end
						
			endcase 
				LUT_free = 1'b 0;
				counter = counter +1;		
			end 	
			
		
	end
	end			
			
			
endmodule
