//-----------------------------------------------------------------------------
//
// Title       : WCHB
// Design      : Asynchronous_logic
// Author      : Rune A. Brnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : WCHB.v
// Generated   : Tue Mar 10 18:42:00 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This module describes the WCHB handshake buffer element used in all switch
//points for the AESRN. C-element instances are included to provide
//stateholding capabilities. The module uses an enable signals instead of 
//the usual acknowledge signal as described in the main report. The interface
//can communicate with all instances using a 4- Phase Dual Rail protocol.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module WCHB (
	
// **** Input wires ****	
	input wire	ev_c_in, 		//Represents the ev_c wire
	input wire  ev_d_in	, 		//Represents the ev_d wire
	input wire	r_e,			//Received enable for the WCHB instance
	input wire	rst,
	
// **** Output registers ****	
	output reg  ev_c_out,		//output ev_c
	output reg	ev_d_out,		//output ev_d
	output reg  t_e	 			//transmitted enable from the WCHB instance
);	

// **** Registers and wires **** 

	wire c_elc_out;     		//C-element for ev_c wire
	wire c_eld_out;				//C-element for ev_d wire
	
// - instanciating two C_elements for the buffer -
	c_element  C_EL1(ev_c_in,r_e,c_elc_out);
	
	c_element  C_EL2(ev_d_in,r_e,c_eld_out);
	
	
	always @ (c_elc_out or c_eld_out or rst ) //asynchronous
		begin 		  							  
			if (rst == 0)	//active low reset
				begin
				ev_c_out <= 0;
				ev_d_out <= 0;
				t_e	<= 0;   //instancated as busy
				end
		    else
				begin
//T_e uses the inverse of the C-element output
//The DR convention prevents "0" on both inputs
//to be coded as a valid value.	
// Added dealy is just for simulation purposes, especially considering
//pipeline distribution.
		 		t_e <= #5 !c_elc_out & !c_eld_out;
				ev_c_out <= #5 c_elc_out;
				ev_d_out <= #5  c_eld_out;
				end
		end;

endmodule
