//-----------------------------------------------------------------------------
//
// Title       : copy_element_test
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : copy_element_test.v
// Generated   : Mon Apr 20 13:39:02 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// SImple test for the output copy element. A configuration is read from file
//and the copy element is configured accordingly.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module copy_element_test ();
	
	parameter ClockPeriod = 20;

// **** Input registers to the test module ****	
	reg [3:0] configuration_bits;
	reg [3:0]  ev_c_in_CE;
	reg [3:0]  ev_d_in_CE;
	reg [7:0]  t_e_test;
	reg rst;
	reg clk;
// **** Output wires to the test module ****	
	wire [7:0]  ev_c_out_CE;
	wire [7:0]  ev_d_out_CE;
	wire [3:0]  r_e_test;

	reg [4:1] Test_vectors[1:1];	//Configuration vector for the Copy Element
	
// - Instanciating test module - 	
	copy_element
		copy_element_test (
			.configuration_bits			(configuration_bits),
			.ev_c_in_CE					(ev_c_in_CE),
			.ev_d_in_CE					(ev_d_in_CE),
			.r_e_CE						(t_e_test),
			.ev_c_out_CE				(ev_c_out_CE),
			.ev_d_out_CE				(ev_d_out_CE),
			.t_e_CE						(r_e_test)
		);

initial clk = 1;
initial rst = 1;
 always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			 
			 $readmemb("CopyElementConf.vec",Test_vectors); //read input vectors
// - Initial values of input signals to copy element under test -
			 clk = 0;
			 rst = 0;
			 t_e_test = 4'b 0000;
			 ev_c_in_CE = 4'b 0000;
			 ev_d_in_CE = 4'b 0000;
// - Reads the first testvector set from the CopyElementConf.vec file. -

			 configuration_bits = Test_vectors[1];
			 
		 	# ClockPeriod rst = 1;		 
			 			 
end

always @ (posedge clk)
	begin: TEST_STIMULUS
	
	// - Local registers -
	reg	 [3:0] CE_input_free;		//r_e_test
	reg  [7:0] wchb_notify;			//t_e_test
	reg	 [7:0] module_free = 8'b 11111111;
	
	integer i,n,m,k,l,o;    //For couting purposes
	
	if (rst)
		begin
//CEs which are ready to receive input are marked			
			for( i = 0; i < 8; i = i+1)
					begin: CHECK_WCHBs
						if( r_e_test[i] == 1 )	
							CE_input_free[i] = 1'b 1;			
						else
							CE_input_free[i] = 1'b 0;
					end	
// - Test sequence with signals issued on output 2 from the peripheral device,
// forking to the branches 2 and 6 for the individual channels -
			if (CE_input_free[2] == 1 )	   
				begin
					ev_c_in_CE[2] = 1;
					ev_d_in_CE[2] = 1;
				end 

// -Test sequence with forking from peripheral line 2 to wires 4 and 5 -
			if (ev_c_out_CE[2] == 1 & ev_c_out_CE[6] == 1) 
	        	begin
					t_e_test[2] = 0;
					t_e_test[6] = 0;
					module_free[2] = 0;
					module_free[6] = 0;
				end 	  

// - When the rest tokens have propagated to the output of each channel,
//the handshake is completed. -
if ( ev_c_out_CE[2] == 0 & ev_d_out_CE[6] == 0 )
	begin
		if (module_free[2] == 0 & module_free[6] == 0 )
			begin
				t_e_test[2] = 1;
				t_e_test[6] = 1;
				module_free[2] = 1;	  
				module_free[6] = 1;
			end
	end				
				
				
// - Setting data outputs to zero -
			if ( r_e_test[2] == 0 )
				begin
					ev_c_in_CE[2] = 0;
					ev_d_in_CE[2] = 0;
				end 		
			
// - For each clock cycle the transmitted enable signal from the testbench
//is updated. -
				for( n = 0; n < 8; n = n+1)
					begin: SET_AVAILABILITY
						if ( module_free[n] == 1 )
							begin
								t_e_test[n] = 1'b 1;	
							end
					end	
			
			
			end
	end
		
endmodule
