//-----------------------------------------------------------------------------
//
// Title       : wchb_handshake_network
// Design      : Asynchronous_logic
// Author      : pcdemon
// Company     : NTNU
//
//-----------------------------------------------------------------------------
//
// File        : wchb_handshake_network.v
// Generated   : Wed Mar 11 16:14:55 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
//	This is a network consisting of 4 buffers involved in a handhshake procedure
// The input signals are Dual-Rail inputs to the first WCHB, and the output
//signals are from the last WCHB. The rest of the signals are internal wires
//to provide the correct handshaking mechanisms. The notation used for the
// signals are correspondent to the signal names associated with the
//event system wires ev_c and ev_d. T_e wires are always transmitted from
// a WCHB instance to the WCHB sending data, while R_e wires are received from
//the WCHB data are beeing sent to.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

//Input/output corresponds to the first and last WCHB instance
module wchb_handshake_network (Ev_ci, Ev_di,R_e, rst,Ev_co,Ev_do,T_e);

	input Ev_ci,Ev_di,R_e,rst;
	output Ev_co, Ev_do, T_e;
	
	//registers for internal input signals to each WCHB 
	
			
	//Wires for internal output signals between each WCHB
	wire  	Ev_co1,Ev_co2,Ev_co3,Ev_do1,Ev_do2,Ev_do3,
			T_e1,T_e2,T_e3,T_e,R_e;

	//Initializing the WCHB instances
	
	WCHB wchb_1 (Ev_ci, Ev_di, T_e1, rst, Ev_co1, Ev_do1, T_e);

    WCHB wchb_2 (Ev_co1, Ev_do1, T_e2, rst, Ev_co2, Ev_do2,T_e1);
	
	WCHB wchb_3 (Ev_co2, Ev_do2, T_e3, rst, Ev_co3, Ev_do3,T_e2);
	
	WCHB wchb_4 (Ev_co3, Ev_do3, R_e, rst, Ev_co, Ev_do,T_e3);
	

endmodule
