//-----------------------------------------------------------------------------
//
// Title       : wchb_testb
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : wchb_testb.v
// Generated   : Tue Mar 10 19:16:21 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
//	Simple testbench to verify the behaviour of the WCHB buffer element
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module wchb_testb (); 
	
	parameter ClockPeriod = 20;
	
// **** Input registers ****
reg clk, ev_c_in, ev_d_in, t_e_test, rst;

// **** Output wires ****
wire ev_c_out, ev_d_out, r_e_test; 		

// - instantiated module -
WCHB WCHB_t(
	.ev_c_in	(ev_c_in),
	.ev_d_in	(ev_d_in),
	.r_e		(t_e_test),
	.rst		(rst),
	.ev_c_out	(ev_c_out),
	.ev_d_out	(ev_d_out),
	.t_e		(r_e_test)
);

initial clk = 1;
initial rst = 1;
 always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			 clk = 1'b 0;
			 rst = 1'b 0;
			 ev_c_in = 1'b 0; ev_d_in = 1'b 0; t_e_test = 1'b 0; //initial values	
			 
				 
			# ClockPeriod rst = 1'b 1;	 
	end
	
always @ (posedge clk)
	begin: Test
		
		integer counter = 0;
		reg notify_test = 1'b 0;
		reg test_free = 1'b 1;
	
	if(rst)
		begin 
// Different test scenarios. When r_e_test is "1" the WCHB is ready
// to receive test input. 
			if (r_e_test == 1 & counter < 5)
				begin
					casez (counter )
						0:
							begin
								ev_c_in <= 1'b 1;
								ev_d_in <= 1'b 0;
							end
						1:
							begin
								ev_c_in <= 1'b 0;
								ev_d_in <= 1'b 1;
							end	
						2:
							begin
								ev_c_in <= 1'b 1;
								ev_d_in <= 1'b 1;
							end
						3:
							begin
								ev_c_in <= 1'b 0;
								ev_d_in <= 1'b 0;
							end
						4:
								begin
								ev_c_in <= 1'b 1;
								ev_d_in <= 1'b 0;
							end	
					endcase	
					
					counter = counter +1;
					notify_test = 1;
				end
				
			if (r_e_test == 0 & notify_test == 1)
				begin		 
//Exclude the following if statement for error free operation. An error
//is inserted when counter = 3 in the form of a missing reset token,
//which will cause the WCHB to halt. 
					if (counter != 3)	
						begin 			// To provoke a handshake error
							ev_c_in <= 1'b 0;
							ev_d_in <= 1'b 0;
					    	notify_test = 1'b 0;
					end
				end
// Sets the testbench ready to receive new inputs, after it has issued
//the required {00} spacer to the WCHB.
			if (ev_c_out == 0 & ev_d_out == 0)
				begin
					if (test_free == 0)
						begin
							t_e_test <= 1'b 1;
							test_free = 1'b 1;	
						end	
				end	
// The testbench receives datat from the WCHB instance, and is set to busy.				
			if(ev_c_out == 1 | ev_d_out == 1)
				begin
					t_e_test <= 1'b 0;
					test_free = 1'b 0;
				end	
// REady to receive new WCHB data				
			if (test_free == 1)
				begin
					t_e_test <= 1'b 1;
				end
		end		

			
end
			//after receiving the negative event "01", an invalid condition is tested
endmodule
