//-----------------------------------------------------------------------------
//
// Title       : AGERN_port_test
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : AGERN_port_test.v
// Generated   : Thu Apr 16 11:29:11 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This module reads configuration vectors from file, configures the AGERN
// routing module, and distributes events according to the configuration file.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module AGERN_port_test ();
	
	
	parameter ClockPeriod = 20;

// **** Input registers to the test module ****	
	reg [255:0] configuration_bits;
	reg [47:0]  ev_c_in;
	reg [47:0]  ev_d_in; 
	reg [47:0]  t_e_test;
	reg rst;
	reg clk;

// **** Output wire from the test module ****	
	wire [47:0]  ev_c_out;
	wire [47:0]  ev_d_out; 
	wire [47:0]  r_e_test;
	

// - Register memory to hold input data from file -
// - Each configuration vector is 32 bits, one for each switch block -

reg [32:1] Conf_vectors[8:1];	

// - Connecting test bench signals to the AGERN module
	AGERN_port
		AGERN_port_instance (
		.configuration_bits			(configuration_bits),
		.ev_c_in					(ev_c_in),
		.ev_d_in					(ev_d_in),
		.r_e						(t_e_test),	//transmitted to AGERN_port module
		.rst						(rst),
		.ev_c_out					(ev_c_out),
		.ev_d_out					(ev_d_out),
		.t_e						(r_e_test) //received in testbench
	);
	
initial clk = 1;
initial rst = 1;
 always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			 
			 $readmemb("AGERNConf.vec",Conf_vectors); //read input vectors
			 
// - Initial values of input signals to switch block module -
			 clk = 0;
			 rst = 0;
			 t_e_test = 48'b 000000000000000000000000000000000000000000000000;
			 ev_c_in = 48'b 000000000000000000000000000000000000000000000000;
			 ev_d_in = 48'b 000000000000000000000000000000000000000000000000;
			 
// - Reads the initial configuration from the file AGERNConf.vec -
// One line in the configuration file configures one switch block
			configuration_bits[31:0] = Conf_vectors[1];
			configuration_bits[63:32] = Conf_vectors[2];
			configuration_bits[95:64] = Conf_vectors[3];
			configuration_bits[127:96] = Conf_vectors[4];
			configuration_bits[159:128] = Conf_vectors[5];
			configuration_bits[191:160] = Conf_vectors[6];
			configuration_bits[223:192] = Conf_vectors[7];
			configuration_bits[255:224] = Conf_vectors[8];
			 
		 	# ClockPeriod rst = 1;
		 			 			 
		end	

always @ (posedge clk)
	begin: TEST_STIMULUS
	
	// - Local registers -
	reg	 [47:0] wchb_free;			//r_e_test
	reg  [47:0] wchb_notify;		//t_e_test
	reg	 [47:0] module_free = 48'b 111111111111111111111111111111111111111111111111;
	integer i,n,m,k,l,o;    //For couting purposes
	
		if (rst)
			begin
// - The first loop iterates through all t_e signals from all WCHBs (r_e to
// the testbench),, and marks the ones that are free for data transfer in
// each of the switch block input directions -
				for( i = 0; i < 48; i = i+1)
					begin: CHECK_WCHBs
						if( r_e_test[i] == 1 )	
							wchb_free[i] = 1'b 1;			
						else
							wchb_free[i] = 1'b 0;
					end	
					
// - This loop iterates through all of the wchb_free variables in each of
//the switchs input directions, and sends a positive event bit on all
// free transmission paths -
				for ( o = 0; o < 48; o = o+1 )
					begin: SEND_DATA
						if ( wchb_free[o] == 1 )
							begin
								ev_c_in[o] = 1'b 1;
								ev_d_in[o] = 1'b 0;
								wchb_notify[o] = 1'b 1;
							end
					 end
					 
// - An event bit is send through the switch, and received by the testbench.
//To make a correct handshake, the TB sets its transmitted enable signal
//to "0", letting the sending WCHB know it is busy. -
				for( m = 0; m < 48; m = m+1 )
					begin: START_HS
						if  (ev_c_out[m] == 1 | ev_d_out[m] == 1 )
							begin
								t_e_test[m] = 1'b 0;
								module_free[m] = 1'b 0;
							end
					end	
					
// -The WCHB which has issued an event bit to the TB is sent the appropriate
// spacer in order to complete the handshake. This spacer means resetting
// for another reception.
				for (k = 0; k < 48; k = k+1 )
					begin: SET_SPACER
						if ( r_e_test[k] == 0 & wchb_notify[k] == 1)
							begin
								ev_c_in[k] = 1'b 0;
								ev_d_in[k] = 1'b 0;
								wchb_notify[k] = 1'b 0;
							end
						
					end
					
// - The final handshake procedure is issued in this loop, where the testbench
//sets its transmitted enable signal to "1" if a spacer is received from
//the correct WCHB element. The handshake is complete.
				for (l = 0; l < 48; l = l+1 ) 
					begin: RESET_HS
						if ( ev_c_out[l] == 0 & ev_d_out[l] == 0 )
							begin
								if( module_free[l] == 0 )
									begin
										t_e_test[l] = 1'b 1;
										module_free[l] = 1'b 1;
									end
							end
					end
					
// - For each clock cycle the transmitted enable signal from the testbench
//is updated. -
				for( n = 0; n < 48; n = n+1)
					begin: SET_AVAILABILITY
						if ( module_free[n] == 1 )
							begin
								t_e_test[n] = 1'b 1;	
							end
					end		
			end			
	end
		
endmodule
