//-----------------------------------------------------------------------------
//
// Title       : ALFERN_module_test
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU
//
//-----------------------------------------------------------------------------
//
// File        : ALFERN_module_test.v
// Generated   : Wed Apr 22 10:39:50 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// Test module for the ALFERN distribution system, also testing LUT
//functionalities in the TCCFB module. All instances are programmed from file.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module ALFERN_module_test ();
	
	parameter ClockPeriod = 20;
	
	reg [83:0] configuration_bits;
	reg [27:0] ev_c_in_module;
	reg [27:0] ev_d_in_module;
	reg [31:0] t_e_test;
 	reg		   rst;
	reg 	   clk;
	 
	wire [31:0] ev_c_out_module;
	wire [31:0] ev_d_out_module;
	wire [27:0] r_e_test;
	
	reg [83:0] conf_temp;
	reg [84:1] Conf_vector[1:1];
	
	initial clk = 1;
	initial rst = 1;
		
ALFERN_port
	ALFERN_module_inst (
		.configuration_bits			(configuration_bits),
		.ev_c_in_module				(ev_c_in_module),
		.ev_d_in_module				(ev_d_in_module),
		.r_e_module					(t_e_test),
		.rst						(rst),
		.ev_c_out_module			(ev_c_out_module),
		.ev_d_out_module			(ev_d_out_module),
		.t_e_module					(r_e_test)
	);
		
 always  
	 #(ClockPeriod / 2) clk = ! clk;
	 
	 initial 
		 begin
			clk = 0;
			 rst = 0;
			 t_e_test = 32'b 0000000000000000000000000000000;
			 ev_c_in_module = 28'b 0000000000000000000000000000;
			 ev_d_in_module = 28'b 0000000000000000000000000000;
			 
			 
			 $readmemb("ALFERNConf.vec",Conf_vector); //read input vectors
			 
			 conf_temp = Conf_vector[1]; //COnfiguration for ALFERN read from file
			 
			 configuration_bits[31:0] = conf_temp[31:0];
			 configuration_bits[63:32] = conf_temp[63:32];
			 configuration_bits[67:64] = conf_temp[67:64];
			 configuration_bits[71:68] = conf_temp[71:68];
			 configuration_bits[75:72] = conf_temp[75:72];
			 configuration_bits[83:76] = conf_temp[83:76];
			 
			 # ClockPeriod rst = 1;
			 
			end 
always @ (posedge clk)
	begin: TEST_STIMULUS
	
	// - Local registers -
	reg	 [27:0] module_free = 28'b 0000000000000000000000000000;			//r_e_test
	reg  [27:0] notify_test = 28'b 0000000000000000000000000000;			//t_e_test;		//t_e_test
	reg	 [27:0] test_free = 28'b 1111111111111111111111111111;
	integer i,n,m,k,l,o;    //For couting purposes
	
		if (rst)
			begin						
				
				 for (o = 0; o <  28; o = o+1 )
				   begin: CHECK_STATUS
					   if( test_free[o] == 1)
						   begin
							   if (o > 15 & o < 24 )
								   begin
									   t_e_test[o+8] = 1'b 1;
									   t_e_test[o] = 1'b 1;
								   end
							else
								t_e_test[o] = 1'b 1;
								
							end
							   
					end	
				
				
// - For each iteration all modules that are transmitting the enable
//signals making them available for data sending will be marked as free.
				for( i = 0; i < 28; i = i+1)
					begin: CHECK_INST
						if( r_e_test[i] == 1 )
							begin
							module_free[i] = 1'b 1;			
							end
						else  
							begin
							module_free[i] = 1'b 0;
							end
					end				
//The test setup is manually programmed.				 	
				if (module_free[14] == 1 & module_free[21] == 1 & module_free[23] == 1 ) 
					begin
					ev_c_in_module[14] = 1'b 1;	   //Selected AGERN input
					ev_d_in_module[14] = 1'b 0;
					notify_test[14] = 1'b 1;
					ev_c_in_module[21] = 1'b 0; 	//ALFERN Inputs
					ev_d_in_module[21] = 1'b 1;
					ev_c_in_module[23] = 1'b 1;
					ev_d_in_module[23] = 1'b 0;
					notify_test[21] = 1'b 1;
					notify_test[23] = 1'b 1;
					end
				
				//if (module_free[21] == 1 & module_free[23] == 1 ) ALFERN inputs
//					begin
//						ev_c_in_module[21] = 1'b 1;
//						ev_d_in_module[21] = 1'b 0;
//						ev_c_in_module[23] = 1'b 1;
//						ev_d_in_module[23] = 1'b 0;
//						notify_test[21] = 1'b 1;
//						notify_test[23] = 1'b 1;
//					end	
					
// -  Setting the correct handshake while providing a compensation for the fact
//that there are more inputs than outputs to the module. This is due to the
//copy elements.
				for (n = 0; n <  28; n = n+1 ) 
	            	begin: HANDSHAKE
						if( r_e_test[n] == 0 & notify_test[n] == 1)
							begin							
										ev_c_in_module[n] = 1'b 0;
										ev_d_in_module[n] = 1'b 0;
										notify_test[n] = 1'b 0;
							end
					end
	//Handshake is provided with the correct offset to account for Copy element
	//forking. This means that the outputs between 15 and 23 are controlled
	//by the same handshake signals as the ones from [31:24]. This is in accord
	//with the copy element forking protocol.
				for (m = 0; m <  32; m = m+1 ) 
	            	begin: END_HS
						if ( ev_c_out_module[m] == 0 & ev_d_out_module[m] == 0)
							begin
								if (m > 23)
									begin
										if (test_free[m-8] == 0 )
											begin
												t_e_test[m] = 1'b 1;
												test_free[m-8] = 1'b 1;
											end
									end
								else if (m <= 15)
									begin
										if(test_free[m] == 0)
											begin
												t_e_test[m] = 1'b 1;
												test_free[m] = 1'b 1;
											end
									end
							end
					end
	//Outputs are received in the testbench, and apropriate enable signals must
	//be set to "0". 
					
			   for (k = 0; k <  32; k = k+1 ) 
				 	begin: INITIATE_HS
						if (ev_c_out_module[k] == 1 | ev_d_out_module[k] == 1 )
							begin
								if ( k > 23 )
									begin
										t_e_test[k] = 1'b 0;
										t_e_test[k-8] = 1'b 0;
										test_free[k-8] = 1'b 0;			
									end
									
								else if (k <= 15)
									begin
										t_e_test[k] = 1'b 0;
										test_free[k] = 1'b 0;		
									end		
							end   
					end
					
			  			
			end		
		end			
			
endmodule
