//-----------------------------------------------------------------------------
//
// Title       : ALFERN_port
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : ALFERN_port.v
// Generated   : Wed Apr 22 08:48:30 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This module connectes the ports with the ALFERN, and also distributes the
///global signals to the AGERN. This description is at a Port / TCCFB level meaning
//that the described connections are between one Port I/O instance and ta
//Timer Counter Combined Functional Block. Described are both local and
//global connections. In this case the timer counter is represented as a
//LUT instance, only providing LUT functionality. The Port / I/O is described
///thorugh the test bench, only issuing events for test pruposes initially.
//The only inputs are connected to a Port / I/O instance. 
//The global switches originally depicted on the system sketches is replaced
//by a cross switch for simplicity of connection.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module ALFERN_port (
	input wire [83:0] configuration_bits, 	//Dependent on number of LUTs in TCCFB
	input wire [27:0] ev_c_in_module,		// See adress table for details
	input wire [27:0] ev_d_in_module,		// See adress table for details
	input wire [31:0] r_e_module,
	input wire		  rst,
	
	output reg [31:0] ev_c_out_module,		//See adress table for details
	output reg [31:0] ev_d_out_module,		//See adress table for details
	output reg [27:0] t_e_module
	
	); 
	
// **** Registers and wires for internal connection ****
	
// - Registers and wires for the Output copy elements -
reg [3:0] r_e_CEin1;
reg [3:0] r_e_CEin2;
reg [7:0] r_e_CE_ALFin1;
reg [7:0] r_e_CE_ALFin2;


reg [3:0] c_in_ALFTC;
reg [3:0] d_in_ALFTC;
wire [7:0] c_out_ALFTC;
wire [7:0] d_out_ALFTC;
wire [3:0] t_e_ALFTC;

wire [7:0] c_out_ALFin1;
wire [7:0] d_out_ALFin1;
wire [7:0] c_out_ALFin2;
wire [7:0] d_out_ALFin2;
wire [3:0] t_e_ALFin1;
wire [3:0] t_e_ALFin2;
wire [3:0] t_e_CEin1;
wire [3:0] t_e_CEin2;

// - Registers and wires for AGERN input/output switch blocks -

wire [7:0] ev_c_out_TC;	
wire [7:0] ev_d_out_TC;
wire [7:0] t_e_swi_S;
wire [7:0] t_e_swo_S;
wire  [7:0] swo_ev_c_out;
wire  [7:0] swo_ev_d_out;
wire  [7:0] swo_t_e;
wire  [7:0] swi_ev_c_out;
wire  [7:0] swi_ev_d_out;
wire  [7:0] swi_t_e;


// - Registers and wires for the TCCFB connections -
reg [1:0] t_e_ALF_TC;
reg [15:0] c_in_TCCFB;
reg [15:0] d_in_TCCFB;
reg [9:0]  TCCFB_r_e;
reg [7:0] ev_c_out_TCCFB;  				//To the AGERN out switch
reg [7:0] ev_d_out_TCCFB;
reg [7:0] t_e_AG_TCCFB;					//TCCFB AGERN enable

wire [9:0] c_out_TCCFB;
wire [9:0] d_out_TCCFB;
wire [15:0] TCCFB_t_e;							
wire [1:0] TCCFB_c_out_ALF;
wire [1:0] TCCFB_d_out_ALF;
wire [3:0] t_e_ALF_inH;
wire [3:0] t_e_ALF_inL;

// - Registers and wires for Dummy connections to unused switch in/outs
wire [7:0] t_e_switch;
wire [7:0] ev_d_out_switch;
wire [7:0] ev_c_out_switch;


always @ *
	begin: SET_VALUES
// - AGERN switches -		
		ev_c_out_module[15:8] = swo_ev_c_out;			//To AGERN switches
		ev_d_out_module[15:8] = swo_ev_d_out;
		t_e_module[7:0] = swo_t_e;						//TO I/O port
		ev_c_out_module[7:0] = swi_ev_c_out;			//TO I/O port
		ev_d_out_module[7:0] = swi_ev_d_out;
			t_e_module[15:8] = swi_t_e;		       		//To AGERN Switches
			
// - ALFERN copy element 1 -
		r_e_CE_ALFin1[3:0] = r_e_module[27:24];	 		//From Port I/O
		r_e_CE_ALFin1[7:4] = TCCFB_t_e[11:8];			//From TCCFB		
		ev_c_out_module[27:24] = c_out_ALFin1[3:0];	  	//To Port / I/O		
		ev_d_out_module[27:24] = d_out_ALFin1[3:0];
		t_e_module[19:16] = t_e_ALFin1;		  			//To module enable
		                                        		
// -ALFERN copy element 2 -
		r_e_CE_ALFin2[3:0] = r_e_module[31:28];	 		//From Port I/O 
		r_e_CE_ALFin2[7:4] = TCCFB_t_e[15:12];	 		//From TCCFB
		ev_c_out_module[31:28] = c_out_ALFin2[3:0];
		ev_d_out_module[31:28] = d_out_ALFin2[3:0];
		t_e_module[23:20] = t_e_ALFin2;	    			//To Module enable
		
// -TCCFB copy element for output -
		c_in_ALFTC[1:0] = ev_c_in_module[25:24];	// From Port I/O
	   	c_in_ALFTC[3:2] = c_out_TCCFB[9:8];	 		//Output from TCCFB
		d_in_ALFTC[1:0] = ev_d_in_module[25:24];
	   	d_in_ALFTC[3:2] = d_out_TCCFB[9:8];			//From TCCFB
		ev_c_out_module[23:16] = c_out_ALFTC;		//From input copy element
		ev_d_out_module[23:16] = d_out_ALFTC;
		t_e_module[25:24] = t_e_ALFTC[1:0];			//From input copy element
		t_e_ALF_TC = t_e_ALFTC[3:2]; 
		
// - For TCCFB module -
	c_in_TCCFB[7:0] = ev_c_out_TC;			     //From the AGERN in switch
	c_in_TCCFB[11:8] = c_out_ALFin1[7:4];		 //From the forked output of CElow
	c_in_TCCFB[15:12] = c_out_ALFin2[7:4];		 //From the forked output of CEhigh
	d_in_TCCFB[7:0] = ev_d_out_TC;
	d_in_TCCFB[11:8] = d_out_ALFin1[7:4];
	d_in_TCCFB[15:12] = d_out_ALFin2[7:4];
	TCCFB_r_e[7:0] = t_e_swo_S;			 		//From global out switch
	TCCFB_r_e[9:8] = t_e_ALFTC[3:2];	 		//From copy element ALFout
	ev_c_out_TCCFB = c_out_TCCFB[7:0]; 			//To AGERN out switch
	ev_d_out_TCCFB = d_out_TCCFB[7:0];
	t_e_AG_TCCFB = TCCFB_t_e[7:0];				//To AGERN in switch
	
		
	end

switch_block
	switch_block_out (
    	.configuration_bits	(configuration_bits[31:0]),
		.ev_c_in_N			(ev_c_in_module[7:0]),		  //Input from Port I/O pins
		.ev_d_in_N			(ev_d_in_module[7:0]),
		.ev_c_in_S			(ev_c_out_TCCFB),			  //From TCCFB output
		.ev_d_in_S			(ev_d_out_TCCFB),
		.r_e_W				(r_e_module[15:8]),			  //From AGERN switches
		.r_e_E				(t_e_switch),
		.rst				(rst),		 
		.ev_c_out_W			(swo_ev_c_out),				  // TO AGERN switches
		.ev_d_out_W			(swo_ev_d_out),
		.ev_c_out_E			(ev_c_out_switch),
		.ev_d_out_E			(ev_d_out_switch),
		.t_e_N				(swo_t_e),					  //TO Port / I/O
		.t_e_S				(t_e_swo_S)					  // TO TCCFB r_e[7:0]
	);
 
switch_block
	switch_block_in (
    	.configuration_bits	(configuration_bits[63:32]),
		.ev_c_in_N			(ev_c_in_module[15:8]),	//INputs from AGERN switches
		.ev_d_in_N			(ev_d_in_module[15:8]),
		.ev_c_in_S			(ev_c_out_switch),	    //Dummy inputs with zero value
		.ev_d_in_S			(ev_d_out_switch),
		.r_e_W				(r_e_module[7:0]),		//From Port / I/O peripheral
		.r_e_E				(t_e_AG_TCCFB),			//From TCCFB
		.rst				(rst),		 
		.ev_c_out_W			(swi_ev_c_out),
		.ev_d_out_W			(swi_ev_d_out),
		.ev_c_out_E			(ev_c_out_TC),			// TO TCCFB in [7:0]
		.ev_d_out_E			(ev_d_out_TC),
		.t_e_N				(swi_t_e),				//To R_e [15:8]
		.t_e_S				(t_e_swi_S)
	);	
	
copy_element
	copy_element_ALFERNinL (
    	 	.configuration_bits			(configuration_bits[67:64]),
			.ev_c_in_CE					(ev_c_in_module[19:16]),
			.ev_d_in_CE					(ev_d_in_module[19:16]),
			.r_e_CE						(r_e_CE_ALFin1),
			.ev_c_out_CE				(c_out_ALFin1),
			.ev_d_out_CE				(d_out_ALFin1),
			.t_e_CE						(t_e_ALFin1)
		);
		
copy_element
	copy_element_ALFERNinH (
    	 	.configuration_bits			(configuration_bits[71:68]),
			.ev_c_in_CE					(ev_c_in_module[23:20]),
			.ev_d_in_CE					(ev_d_in_module[23:20]),
			.r_e_CE						(r_e_CE_ALFin2),
			.ev_c_out_CE				(c_out_ALFin2),
			.ev_d_out_CE				(d_out_ALFin2),
			.t_e_CE						(t_e_ALFin2)
		);

copy_element
	copy_element_TCCFBin (
    	 	.configuration_bits			(configuration_bits[75:72]),
			.ev_c_in_CE					(c_in_ALFTC),
			.ev_d_in_CE					(d_in_ALFTC),
			.r_e_CE						(r_e_module[23:16]),
			.ev_c_out_CE				(c_out_ALFTC),
			.ev_d_out_CE				(d_out_ALFTC),
			.t_e_CE						(t_e_ALFTC)
		); 
		
TCCFB
	TCCFB_inst (
		.TCCFB_configuration				(configuration_bits[83:76]),
		.ev_c_in_TCCFB						(c_in_TCCFB),
		.ev_d_in_TCCFB						(d_in_TCCFB),
		.r_e_TCCFB							(TCCFB_r_e),
		.rst								(rst),
		.ev_c_out_TCCFB						(c_out_TCCFB),
		.ev_d_out_TCCFB						(d_out_TCCFB),
		.t_e_TCCFB							(TCCFB_t_e)
		
	);
		
	
		
endmodule
