//-----------------------------------------------------------------------------
//
// Title       : TCCFB
// Design      : Asynchronous_logic
// Author      : Rune A. Bjrnerud
// Company     : NTNU / Atmel Norway
//
//-----------------------------------------------------------------------------
//
// File        : TCCFB.v
// Generated   : Tue Apr 21 18:05:30 2009
// From        : interface description file
// By          : Itf2Vhdl ver. 1.21
//
//-----------------------------------------------------------------------------
//
// Description : 
// This is an example module of a TCCFB using CCbuf registers as LUTs.
// Signals can be received both from the ALFERN and the AGERN, and sent
//on the same routing networks. Only ONE 3-LUT is included internally in 
//this example. Each test-scenario for the TCCFB and LUT block must be
//specified manually, and two configuration examples are included. The TCCFB
//Interacts as the interface between the impolemented 3-LUT and surrounding 
//AGERN / ALFERN distribution units. The 4-PDR handshake protocol is
//implemented as a part of the LUT, not the TCCFB for this version.
//-----------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module TCCFB (
// **** Input wires ****  

//- The amount of configuration bits may vary depending on how many registers
//the TCCFB uses for LUT functionality -
	
	input wire [7:0]  TCCFB_configuration, 
	input wire [15:0] ev_c_in_TCCFB, 	//[15:8]  alfern, [7:0] AGERN
	input wire [15:0] ev_d_in_TCCFB,
	input wire [9:0]  r_e_TCCFB, 		//[9:8] ALFERN, [7:0] AGERN
	input wire        rst,

// **** Output registers ****
	output reg [9:0] ev_c_out_TCCFB,	//[9:8] ALFERN, [7:0] AGERN
    output reg [9:0] ev_d_out_TCCFB,
	output reg [15:0] t_e_TCCFB		//[15:8]  ALFERN, [7:0] AGERN
	);
	
//Handling the reset conditions for the TCCFB. The reset function will only
//reset the outputs NOT connected to LUTs inside the TCCFB. These outputs
//are reset accordingly by the LUT.

reg [2:0] inputs_c_LUT;
reg [2:0] inputs_d_LUT;
wire ev_c_o_LUT;
wire ev_d_o_LUT;
wire [2:0] t_e_LUT;

//The values in this section must be set manually for each test scenario,
//and must be set according to the adress table defined for the TCCFB.
always @ *
	begin: SET_VALUES
		inputs_c_LUT[0] = ev_c_in_TCCFB[14];	 //0 replaced by 14 for scen 2
		inputs_c_LUT[1] = ev_c_in_TCCFB[10];	 //1 replaced by 10	for scen 2
		inputs_c_LUT[2] = ev_c_in_TCCFB[8];
		inputs_d_LUT[0] = ev_d_in_TCCFB[14];
		inputs_d_LUT[1] = ev_d_in_TCCFB[1];
		inputs_d_LUT[2] = ev_d_in_TCCFB[10];
	

		t_e_TCCFB[14] = t_e_LUT[0];
		t_e_TCCFB[10] = t_e_LUT[1];
		t_e_TCCFB[8] =  t_e_LUT[2];		   //	Local ALFERN connection
		
		ev_c_out_TCCFB[8] = ev_c_o_LUT;	   //Output [7:0] AGERN, [9:8] ALFERN CE
		ev_d_out_TCCFB[8] = ev_d_o_LUT;
		
	end
		
always @ (rst)
	begin: RESET
		if (!rst)
			begin	
								
//************Test scenario 2  ************				
				ev_c_out_TCCFB[7:0] = 8'b 00000000; 
				ev_c_out_TCCFB[9] = 1'b 0; 
				
				ev_d_out_TCCFB[7:0] = 8'b 00000000; 
				ev_d_out_TCCFB[9] = 1'b 0;

 

//- Because the LUT gives handshake signals to its own inputs, these must be
//omitted in the TCCFB handshake signaling.	

//************Test scenario 1 ************
				//t_e_TCCFB[5:2] = 4'b 0000; //Channel 0 and 1 reset by LUT.
//				t_e_TCCFB[7:6] = 2'b 00; 		  //Channel 8 set by LUT
//				t_e_TCCFB[15:9] = 1'b 0; 

//************Test scenario 2 ************
				t_e_TCCFB[7:0] = 8'b 00000000; 
				t_e_TCCFB[9] = 1'b 0; 
				t_e_TCCFB[13:11] = 3'b 000;
				t_e_TCCFB[15] = 1'b 0;
				
				//t_e_TCCFB[7:1] = 7'b 0000000; 
				//t_e_TCCFB[11:9] = 3'b 000; 		  
				//t_e_TCCFB[15:13] = 3'b 000;
				
				
			end						 
		else //Must be set to "1" for proper reset
			begin	
//************Test scenario 1 ************
				//t_e_TCCFB[5:2] = 4'b 1111; //Channel 0 and 1 reset by LUT.
//				t_e_TCCFB[7:6] = 2'b 11; 		  //Channel 8 set by LUT
//				t_e_TCCFB[15:9] = 1'b 1;


//************Test scenario 2 ************
				t_e_TCCFB[7:0] = 8'b 11111111; 
				t_e_TCCFB[9] = 1'b 1; 
				t_e_TCCFB[13:11] = 3'b 111;
				t_e_TCCFB[15] = 1'b 1;
				
				//t_e_TCCFB[7:1] = 7'b 1111111; 
				//t_e_TCCFB[11:9] = 3'b 111; 		  
				//t_e_TCCFB[15:13] = 3'b 111;
			end		
	end	
	
//-Instantiating connections to the LUT module(s) -
	TCCFB_LUT
		TCCFB_LUT_1 (
			.configuration_bits			(TCCFB_configuration),
			.ev_c_in_LUT				(inputs_c_LUT),
			.ev_d_in_LUT				(inputs_d_LUT),
			.r_e_LUT					(r_e_TCCFB[8]),  //AGERN channel 2 for output
			.rst						(rst),																		 													
			.ev_c_out_LUT				(ev_c_o_LUT),
			.ev_d_out_LUT				(ev_d_o_LUT),
			.t_e_LUT					(t_e_LUT)
		);	
		
	

endmodule
